/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  void AFsetHinfo (const char Info[])
  char *AFgetHinfo (void)

Purpose:
  Set the audio file header information string
  Get the audio file header information string

Description:
  This routine sets the information string to be written to an audio file
  header.  By default the header information records specify the date and user
  name.  This routine allows the user to specify information records that are
  used in addition to, or in place of, the standard header information.  This
  routine must be called before the audio file to be created is first opened
  using AFopenWrite.  AFopenWrite resets the header string to a NULL string.

  Standard Header Information: 
    date: 1994-01-25 19:19:39 UTC    date
    sample_rate: 8012.5              sampling frequency (if non-integer)
    user: kabal@aldebaran            user
    program: CopyAudio               program name (set by UTsetProg)

  Additional structured information records should follow this format; a named
  field terminated by a colon followed by numeric data or a character string.
  Comments can follow as unstructured information.
    Record delimiter: Newline character or the two character escape
        sequence "\" + "n".  The last record need not have a delimiter.
    Line delimiter: Within records, lines are delimiteded by a carriage
        control character, the two character escape sequence "\" + "r",
        or the two character sequence "\" + newline.
  If the information string starts with a record delimiter, the header
  information string is appended to the standard header information.  If not,
  the user supplied header information string appears alone.

  If the input header information string is a NULL pointer, the standard
  information is used.  If the header information string starts with a record
  delimiter, the input header information string is added to the existing
  information records (the standard information and/or any previously
  specified records).  Otherwise the input header information string replaces
  the standard information.  The following examples illustrate the behaviour.

  - AFsetHinfo (NULL)
      Reset, use only the standard information records
  - AFsetHinfo ("")
      No header information records
  - AFsetHinfo ("<info>")
      Replace existing information with <info>; do not use the standard
      information records
  - AFsetHinfo ("\n<info>")
      Add <info> to the existing information string.

  The procedure AFgetHinfo is used internally by the audio file routines to
  retrieve the user supplied header information string.

Parameters:
  AFsetHinfo:
   -> const char Info[]
      String containing the user supplied header information
  AFgetHinfo:
  <-  char *AFgetUinfo
      Pointer to the header information string.  A NULL pointer indicates that
      no information string has been set.

Author / revision:
  P. Kabal  Copyright (C) 2000

  $Revision: 1.36 $  $Date: 2000/11/21 14:52:32 $

-------------------------------------------------------------------------*/

static char rcsid[] = "$Id: AFsetHinfo.c 1.36 2000/11/21 AFsp-v5r2 $";

#include <string.h>

#include <libtsp.h>
#include <libtsp/AFmsg.h>
#include <libtsp/AFpar.h>

static char *Uinfo = NULL;


void
AFsetHinfo (const char Info[])

{
  int Ni;
  static int Ninfo = 0;

  if (Info == NULL) {
    UTfree ((void *) Uinfo);
    Uinfo = NULL;
    Ninfo = 0;
  }
  else {

/* Copy the header information string to an allocated buffer */
    if (Info[0] == '\n') {
      Ni = Ninfo;
      Ninfo += strlen (Info);
      Uinfo = (char *) UTrealloc (Uinfo, Ninfo+1);
      strncpy (&Uinfo[Ni], Info, (size_t) (Ninfo-Ni));
      Uinfo[Ninfo] = '\0';
   }
    else {
      Ninfo = strlen (Info);
      UTfree ((void *) Uinfo);
      Uinfo = (char *) UTmalloc (Ninfo+1);
      strncpy (Uinfo, Info, (size_t) Ninfo);
      Uinfo[Ninfo] = '\0';
    }
  }
}

char *
AFgetHinfo (void)

{
  return Uinfo;
}
