/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  int AFrdI1 (AFILE *AFp, float Fbuff[], int Nreq)

Purpose:
  Read 8-bit integer data from an audio file (return float values)

Description:
  This routine reads a specified number of 8-bit integer samples from an audio
  file.  The data in the file is converted to float format on output.

Parameters:
  <-  int AFrdI1
      Number of data values transferred from the file.  On reaching the end of
      the file, this value may be less than Nreq.
   -> AFILE *AFp
      Audio file pointer for an audio file opened by AFopenRead
  <-  float Fbuff[]
      Array of floats to receive the samples
   -> int Nreq
      Number of samples requested.  Nreq may be zero.

Author / revision:
  P. Kabal  Copyright (C) 2001

  $Revision: 1.14 $  $Date: 2001/03/27 16:33:57 $

-------------------------------------------------------------------------*/

static char rcsid [] = "$Id: AFrdI1.c 1.14 2001/03/27 AFsp-v5r2 $";

#include <libtsp/AFdataio.h>
#include <libtsp/AFpar.h>
#include <libtsp/UTtypes.h>

#define LW		FDL_INT8
#define MINV(a, b)	(((a) < (b)) ? (a) : (b))
#define NBBUF		8192

#define FREAD(buf,size,nv,fp)	(int) fread ((char *) buf, (size_t) size, \
					     (size_t) nv, fp)


int
AFrdI1 (AFILE *AFp, float Fbuff[], int Nreq)

{
  int is, N, i, Nr;
  int1_t Buf[NBBUF/LW];
  float g;

  for (is = 0; is < Nreq; ) {

    /* Read data from the audio file */
    N = MINV (NBBUF / LW, Nreq - is);
    Nr = FREAD (Buf, LW, N, AFp->fp);

    /* Convert to float */
    g = (float) AFp->ScaleF;
    for (i = 0; i < Nr; ++i, ++is)
      Fbuff[is] = g * Buf[i];

    if (Nr < N)
      break;
  }

  return is;
}
